import{s as a,c as s,a as e,C as t}from"./chunk-B4BG7PRW-CT4xgCmp.js";import{_ as i}from"./index-CxVEIQKK.js";import"./chunk-FMBD7UC4-unNibHct.js";import"./chunk-55IACEB6-DNkahd88.js";import"./chunk-QN33PNHL-DD8e4AkV.js";var p={parser:e,get db(){return new t},renderer:s,styles:a,init:i(r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute},"init")};export{p as diagram};
